#include "LCD_driver.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "WProgram.h"
#include "HardwareSerial.h"

//************************************************************************
//					Main Code
//************************************************************************


// Includes
#include <Servo.h> 
#include "TimerThree.h"
// Constants
const int TurnAmount   =1275;
const int StrightAmount=1900;
const int GridSize   =15;
const int RobotWidth = 640;
const float DegToRad = 0.0174532925;
const int numReadings =5;

int scan_one_cell=740;//lower number makes it see further

// Global variables
int NoSteps=0;

// Scanning Variables
unsigned long int scan_angle=750;//position of scanner, in units of servo command
unsigned int tmp=255;
unsigned int object_found=0;

int old_state=1;//records current pointed direction of robot: 1 is up, 2 is right, 3 is down, and 4 is left (clockwise)
int new_state=1;//required direction of robot
//X is vertical, Y is horizontal
int Map[GridSize][GridSize];


Servo myservo;
// Wave Front Variables
const int nothing=0;
const int wall=255;
const int goal=1;
const int robot=254;

//starting robot/goal locations
int robot_x=8;
int robot_y=8;
int goal_y=8;
int goal_x=2;
int cell_size=330;//the size of the robot

//Map locations
int x=0;
int y=0;

//temp variables
int counter=0;

//when searching for a node with a lower value
int minimum_node=250;
int min_node_location=0;
int reset_min=250;//anything above this number is a special item, ie a wall or robot
// Global variables

void setup() 
{ 

  //lcd setup
  ioinit();           //Initialize I/O
  LCDInit();	    //Initialize the LCD
  LCDContrast(44);
  LCDClear(WHITE);    // Clear LCD to a solid color
  LCDPutStr("Welcome to", 32, 4, BLACK, WHITE);
  LCDPutStr("Adaptive", 48, 4, BLACK, WHITE);
  LCDPutStr("Mapping!", 64, 4, BLACK, WHITE);

  //mappiung setup 

  Serial.begin(9600);
  myservo.attach(8);  // attaches the servo on pin 9 to the servo object 
  myservo.write(30);
  Timer3.initialize();
  setupStepper();

  for (int H=0; H<GridSize;H++)
  {
    for (int W=0; W<GridSize;W++)
    {
      Map[H][W]=0;
    }
  }
  //reset encoder function
  //store robot location in Map
  Map[robot_x][robot_y]=robot;
  //store robot location in Map
  Map[goal_x][goal_y]=goal;
  // Drive around until a button or unsafe condition is detected
  delay(2000);
  LCDClear(WHITE);
}


void loop() 
{
  while( Map[robot_x][robot_y]!=goal)//program ends when robot reaches goal
  {
    find_walls();
    new_state=propagate_wavefront();
    //////////////wavefront code//////////////
    while(new_state!=old_state && new_state!=0)
    {
      //if not pointed in the right direction, rotate
      if (abs(old_state - new_state) == 2)//rotate 180 degrees
      {
        rotate_CCW();
        rotate_CW();
      }
      else if ((signed int)(old_state - new_state) == -1 || (signed int)(old_state - new_state) == 3)//rotate 90 degrees CW
        rotate_CW();
      else if ((signed int)(old_state - new_state) == 1 || (signed int)(old_state - new_state) == -3)//rotate 90 degrees CCW
        rotate_CCW();
      //make new state the old state
      old_state=new_state;
      //recheck with new location
      find_walls();
      //find new location to go to
      new_state=propagate_wavefront();
    }
    //there is no solution

    if (new_state==0)
    {
      delay(2500);
      //reset the entire Map
      clear_Map();
    }
    //go to new location
    else//move if there is a solution, otherwise dont move forward
    {
      Map[robot_x][robot_y]=256;
      if (new_state==1)
        robot_x--;
      if (new_state==2)
        robot_y++;
      if (new_state==3)
        robot_x++;
      if (new_state==4)
        robot_y--;
      Map[robot_x][robot_y]=robot;
      straight();//distance, velocity
      //update new location of robot

    }
    //make new state the old state
    if(new_state!=0)
      old_state=new_state;

    if (robot_x==goal_x &&robot_y==goal_y)
    {
      goal_x=8;
      Map[goal_x][goal_y]=goal;
      Map[robot_x][robot_y]=robot;  
      unpropagate();
      UpdateLcd();
    }
  }



}


//rotate 90
void rotate_CCW()
{
  turnRight();
  Timer3.attachInterrupt(MoveTurn, 900);
  UpdateLcd();
  while(NoSteps!=0)
  {
  }
}


//rotate 90
void rotate_CW()
{
  turnLeft();
  Timer3.attachInterrupt(MoveTurn, 900);
  UpdateLcd();
  while(NoSteps!=0)
  {
  }
}


//drive straight function 
void straight()
{
  stright();
  Timer3.attachInterrupt(MoveStright, 900);
  UpdateLcd();
  while(NoSteps!=0)
  {
  }
}

// draws the screen on the map
void UpdateLcd()
{   
  LCDClear(WHITE);
  for (int H=0; H<GridSize;H++)
  {
    for (int W=0; W<GridSize;W++)
    {
      switch (Map[H][W])
      {
      case 256:
        {
          LCDSetRect(H*8+1,W*8+1,H*8+7,W*8+7,1,YELLOW);
        }
        break;
      case wall:
        {
          LCDSetRect(H*8+1,W*8+1,H*8+7,W*8+7,1,RED);
        }
        break;
      case robot:
        { 
          LCDSetRect(H*8+1,W*8+1,H*8+7,W*8+7,1,GREEN);
        }
        break;
      case 1:
        { 
          LCDSetRect(H*8+1,W*8+1,H*8+7,W*8+7,1,BLUE);
        }
        break;
      default:
        { 
          LCDSetRect(H*8+1,W*8+1,H*8+7,W*8+7,1,WHITE);
        } 
        break;
        delay(1);
      }
    }
  }
}




